/**
 * Clean Framework Adapter Manager Implementation
 * @iota-big3/sdk-adapters
 */
/// <reference types="node" />
import { EventEmitter } from 'events';
import { AdapterCapabilities, AdapterConfig, AdapterMetadata, AdapterMetrics, ExpressLikeApp, FastifyLikeApp, FrameworkApp, FrameworkType, HandlerResponse, HealthCheckResult, HTTPMethod, KoaLikeApp, Middleware, MiddlewareContext, RouteHandler, ValidationResult } from './types';
/**
 * Base Framework Adapter Class
 */
export declare abstract class FrameworkAdapter<TConfig extends AdapterConfig = AdapterConfig> extends EventEmitter {
    protected config: TConfig;
    protected middleware: Map<string, Middleware>;
    protected routes: RouteHandler[];
    protected metrics: AdapterMetrics;
    protected isEnabled: boolean;
    protected initialized: boolean;
    abstract readonly framework: FrameworkType;
    abstract readonly version: string;
    abstract readonly metadata: AdapterMetadata;
    abstract readonly capabilities: AdapterCapabilities;
    constructor(config: TConfig);
    private initializeMetrics;
    private initialize;
    abstract applyMiddleware(app: ExpressLikeApp | FastifyLikeApp | KoaLikeApp): Promise<void>;
    abstract registerRoute(app: ExpressLikeApp | FastifyLikeApp | KoaLikeApp, route: RouteHandler): Promise<void>;
    abstract createApp(): Promise<FrameworkApp>;
    protected onInitialize(): Promise<void>;
    protected onDestroy(): Promise<void>;
    getFramework(): FrameworkType;
    checkHealth(): Promise<HealthCheckResult>;
    ensureInitialized(): Promise<void>;
    updateConfig(newConfig: Partial<TConfig>): void;
    getConfig(): TConfig;
    addMiddleware(middleware: Middleware): void;
    removeMiddleware(name: string): boolean;
    getMiddleware(): Middleware[];
    hasMiddleware(name: string): boolean;
    addRoute(route: RouteHandler): void;
    removeRoute(method: HTTPMethod, path: string): boolean;
    getRoutes(): RouteHandler[];
    findRoute(method: HTTPMethod, path: string): RouteHandler | undefined;
    handleRequest(context: MiddlewareContext): Promise<HandlerResponse>;
    protected processRequest(_context: MiddlewareContext): Promise<HandlerResponse>;
    validateRoute(route: RouteHandler): ValidationResult;
    validateMiddleware(middleware: Middleware): ValidationResult;
    healthCheck(): Promise<HealthCheckResult>;
    getMetrics(): AdapterMetrics;
    private emitEvent;
    private handleError;
    private updateAverageResponseTime;
    destroy(): Promise<void>;
    enable(): void;
    disable(): void;
    isAdapterEnabled(): boolean;
}
/**
 * Express Framework Adapter
 */
export declare class ExpressAdapter extends FrameworkAdapter<AdapterConfig> {
    readonly framework: FrameworkType;
    readonly version: string;
    readonly metadata: AdapterMetadata;
    readonly capabilities: AdapterCapabilities;
    private expressModule;
    constructor(config: AdapterConfig);
    createApp(): Promise<ExpressLikeApp>;
    applyMiddleware(app: ExpressLikeApp): Promise<void>;
    registerRoute(app: ExpressLikeApp, route: RouteHandler): Promise<void>;
    private createExpressRouteHandler;
    private createStubRouteHandler;
}
/**
 * Fastify Framework Adapter
 */
export declare class FastifyAdapter extends FrameworkAdapter<AdapterConfig> {
    readonly framework: FrameworkType;
    readonly version: string;
    readonly metadata: AdapterMetadata;
    readonly capabilities: AdapterCapabilities;
    createApp(): Promise<FastifyLikeApp>;
    applyMiddleware(app: FastifyLikeApp): Promise<void>;
    registerRoute(app: FastifyLikeApp, route: RouteHandler): Promise<void>;
}
/**
 * Koa Framework Adapter
 */
export declare class KoaAdapter extends FrameworkAdapter<AdapterConfig> {
    readonly framework: FrameworkType;
    readonly version: string;
    readonly metadata: AdapterMetadata;
    readonly capabilities: AdapterCapabilities;
    createApp(): Promise<KoaLikeApp>;
    applyMiddleware(app: KoaLikeApp): Promise<void>;
    registerRoute(app: KoaLikeApp, route: RouteHandler): Promise<void>;
}
/**
 * Adapter Registry for managing multiple framework adapters
 */
export declare class AdapterRegistry extends EventEmitter {
    private static instance;
    private adapters;
    private factories;
    private isEnabled;
    private constructor();
    static getInstance(): AdapterRegistry;
    private registerDefaultFactories;
    register(adapter: FrameworkAdapter): void;
    unregister(framework: FrameworkType): boolean;
    get<T extends FrameworkAdapter>(framework: FrameworkType): T | undefined;
    create(framework: FrameworkType, config?: AdapterConfig): FrameworkAdapter;
    has(framework: FrameworkType): boolean;
    supports(framework: FrameworkType): boolean;
    getAll(): FrameworkAdapter[];
    getSupportedFrameworks(): FrameworkType[];
    getRegisteredFrameworks(): FrameworkType[];
    healthCheckAll(): Promise<Record<FrameworkType, HealthCheckResult>>;
    clear(): void;
    size(): number;
    enable(): void;
    disable(): void;
    isRegistryEnabled(): boolean;
}
/**
 * Framework Adapter Manager - Main orchestrator
 */
export declare class AdapterManager extends EventEmitter {
    private registry;
    private config;
    private isEnabled;
    constructor(config?: AdapterConfig);
    private setupEventForwarding;
    createAdapter(framework: FrameworkType, config?: AdapterConfig): FrameworkAdapter;
    registerAdapter(adapter: FrameworkAdapter): void;
    getAdapter<T extends FrameworkAdapter>(framework: FrameworkType): T | undefined;
    getSupportedFrameworks(): FrameworkType[];
    getRegisteredFrameworks(): FrameworkType[];
    performHealthCheck(): Promise<Record<FrameworkType, HealthCheckResult>>;
    getOverallHealth(): Promise<HealthCheckResult>;
    updateConfig(newConfig: Partial<AdapterConfig>): void;
    getConfig(): AdapterConfig;
    enable(): void;
    disable(): void;
    isManagerEnabled(): boolean;
    destroy(): Promise<void>;
}
//# sourceMappingURL=adapter-manager.d.ts.map