"use strict";
/**
 * Clean Framework Adapter Manager Implementation
 * @iota-big3/sdk-adapters
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.AdapterManager = exports.AdapterRegistry = exports.KoaAdapter = exports.FastifyAdapter = exports.ExpressAdapter = exports.FrameworkAdapter = void 0;
const events_1 = require("events");
/**
 * Base Framework Adapter Class
 */
class FrameworkAdapter extends events_1.EventEmitter {
    constructor(config) {
        super();
        this.middleware = new Map();
        this.routes = [];
        this.isEnabled = true;
        this.initialized = false;
        this.config = config;
        this.metrics = this.initializeMetrics();
        this.initialize();
    }
    initializeMetrics() {
        return {
            requestsHandled: 0,
            middlewareExecutions: 0,
            routesRegistered: 0,
            errorsEncountered: 0,
            averageResponseTime: 0,
            uptime: Date.now()
        };
    }
    async initialize() {
        try {
            await this.onInitialize();
            this.initialized = true;
            this.emitEvent('adapter-initialized', { framework: this.framework });
        }
        catch (error) {
            this.handleError('Initialization failed', error);
        }
    }
    // Optional lifecycle hooks
    async onInitialize() {
        // Default implementation - can be overridden
    }
    async onDestroy() {
        // Default implementation - can be overridden
    }
    // Public API Methods
    getFramework() {
        return this.framework;
    }
    async checkHealth() {
        return {
            status: 'healthy',
            framework: this.framework,
            version: this.version,
            uptime: Date.now() - this.metrics.uptime,
            metrics: this.metrics,
            details: {
                middlewareCount: this.middleware.size,
                routeCount: this.routes.length,
                enabled: this.isEnabled,
                initialized: this.initialized
            }
        };
    }
    // Public method to ensure initialization
    async ensureInitialized() {
        if (!this.initialized) {
            await this.initialize();
        }
    }
    // Configuration management
    updateConfig(newConfig) {
        this.config = { ...this.config, ...newConfig };
        this.emitEvent('configuration-updated', { config: this.config });
    }
    getConfig() {
        return { ...this.config };
    }
    // Middleware management
    addMiddleware(middleware) {
        if (!this.isEnabled) {
            throw new Error('Adapter is disabled');
        }
        this.middleware.set(middleware.name, middleware);
        this.emitEvent('middleware-added', { middleware: middleware.name });
    }
    removeMiddleware(name) {
        const removed = this.middleware.delete(name);
        if (removed) {
            this.emitEvent('middleware-removed', { middleware: name });
        }
        return removed;
    }
    getMiddleware() {
        return Array.from(this.middleware.values())
            .filter(mw => mw.enabled !== false)
            .sort((a, b) => (a.order || 0) - (b.order || 0));
    }
    hasMiddleware(name) {
        return this.middleware.has(name);
    }
    // Route management
    addRoute(route) {
        if (!this.isEnabled) {
            throw new Error('Adapter is disabled');
        }
        this.routes.push(route);
        this.metrics.routesRegistered++;
        this.emitEvent('route-registered', { route: `${route.method} ${route.path}` });
    }
    removeRoute(method, path) {
        const initialLength = this.routes.length;
        this.routes = this.routes.filter(r => !(r.method === method && r.path === path));
        if (this.routes.length < initialLength) {
            this.emitEvent('route-removed', { route: `${method} ${path}` });
            return true;
        }
        return false;
    }
    getRoutes() {
        return [...this.routes];
    }
    findRoute(method, path) {
        return this.routes.find(r => r.method === method && r.path === path);
    }
    // Request handling
    async handleRequest(context) {
        if (!this.isEnabled) {
            throw new Error('Adapter is disabled');
        }
        const startTime = Date.now();
        try {
            // Execute middleware chain
            const middleware = this.getMiddleware();
            for (const mw of middleware) {
                await mw.handler(context);
                this.metrics.middlewareExecutions++;
            }
            // Update metrics - increment requestsHandled
            this.metrics.requestsHandled++;
            this.updateAverageResponseTime(Date.now() - startTime);
            return {
                status: 'ok',
                message: 'Request processed',
                metrics: this.metrics
            };
        }
        catch (error) {
            this.metrics.errorsEncountered++;
            throw error;
        }
    }
    async processRequest(_context) {
        // Default implementation - can be overridden
        return { status: 'ok', message: 'Request processed' };
    }
    // Validation
    validateRoute(route) {
        const errors = [];
        if (!route.method) {
            errors.push({ field: 'method', message: 'HTTP method is required', code: 'MISSING_METHOD' });
        }
        if (!route.path) {
            errors.push({ field: 'path', message: 'Route path is required', code: 'MISSING_PATH' });
        }
        if (!route.handler) {
            errors.push({ field: 'handler', message: 'Route handler is required', code: 'MISSING_HANDLER' });
        }
        return {
            valid: errors.length === 0,
            errors: errors.length > 0 ? errors : undefined
        };
    }
    validateMiddleware(middleware) {
        const errors = [];
        if (!middleware.name) {
            errors.push({ field: 'name', message: 'Middleware name is required', code: 'MISSING_NAME' });
        }
        if (!middleware.handler) {
            errors.push({ field: 'handler', message: 'Middleware handler is required', code: 'MISSING_HANDLER' });
        }
        return {
            valid: errors.length === 0,
            errors: errors.length > 0 ? errors : undefined
        };
    }
    // Health and monitoring
    async healthCheck() {
        const checks = {
            initialized: this.initialized,
            enabled: this.isEnabled,
            hasRoutes: this.routes.length > 0,
            hasMiddleware: this.middleware.size > 0
        };
        const status = Object.values(checks).every(check => check) ? 'healthy' :
            Object.values(checks).some(check => check) ? 'degraded' : 'unhealthy';
        return {
            status,
            timestamp: new Date(),
            checks,
            metrics: this.getMetrics(),
            framework: this.framework
        };
    }
    getMetrics() {
        return {
            ...this.metrics,
            uptime: Date.now() - this.metrics.uptime
        };
    }
    // Event handling
    emitEvent(type, data) {
        const event = {
            type,
            timestamp: new Date(),
            source: this.config.framework,
            data
        };
        this.emit(type, event);
    }
    // Error handling
    handleError(message, error) {
        this.metrics.errorsEncountered++;
        const errorMessage = error instanceof Error ? error.message : String(error);
        const adapterError = {
            code: 'ADAPTER_ERROR',
            message: `${message}: ${errorMessage}`,
            details: error,
            timestamp: new Date()
        };
        this.emitEvent('error-occurred', adapterError);
    }
    // Utility methods
    updateAverageResponseTime(responseTime) {
        if (this.metrics.requestsHandled === 1) {
            this.metrics.averageResponseTime = responseTime;
        }
        else {
            this.metrics.averageResponseTime =
                (this.metrics.averageResponseTime * (this.metrics.requestsHandled - 1) + responseTime) / this.metrics.requestsHandled;
        }
    }
    // Lifecycle management
    async destroy() {
        try {
            await this.onDestroy();
            this.emitEvent('adapter-destroyed', { framework: this.framework });
            this.removeAllListeners();
        }
        catch (error) {
            this.handleError('Destruction failed', error);
        }
    }
    // Enable/disable functionality
    enable() {
        this.isEnabled = true;
    }
    disable() {
        this.isEnabled = false;
    }
    isAdapterEnabled() {
        return this.isEnabled;
    }
}
exports.FrameworkAdapter = FrameworkAdapter;
/**
 * Express Framework Adapter
 */
class ExpressAdapter extends FrameworkAdapter {
    constructor(config) {
        super(config);
        this.framework = 'express';
        this.version = '1.0.0';
        this.metadata = {
            name: 'express-adapter',
            version: '1.0.0',
            framework: 'express',
            description: 'Express.js framework adapter',
            features: ['middleware', 'routing', 'static-files', 'templating'],
            dependencies: ['express']
        };
        this.capabilities = {
            middleware: true,
            routing: true,
            validation: true,
            errorHandling: true,
            logging: true,
            metrics: true,
            healthChecks: true
        };
        // Try to load Express if available
        try {
            // Try multiple paths to find Express
            try {
                this.expressModule = require('express');
            }
            catch {
                // Try from root node_modules
                this.expressModule = require('/Users/allan/iota-big3-sdk/node_modules/express');
            }
        }
        catch (error) {
            // Express not available, will use stub
            this.expressModule = null;
        }
    }
    async createApp() {
        // If Express is available, create real app
        if (this.expressModule) {
            const express = this.expressModule;
            const app = express();
            // Apply basic configuration
            app.set('trust proxy', this.config.trustProxy || false);
            app.set('x-powered-by', false);
            // Apply default middleware
            app.use(express.json({ limit: this.config.jsonLimit || '10mb' }));
            app.use(express.urlencoded({
                extended: true,
                limit: this.config.urlEncodedLimit || '10mb'
            }));
            // Emit app-created event
            const event = {
                type: 'adapter-event',
                timestamp: new Date(),
                source: this.framework,
                data: { type: 'app-created', framework: this.framework }
            };
            this.emit('adapter-event', event);
            return app;
        }
        // Fallback to stub implementation
        const app = {
            use: (_handler) => { },
            get: (_path, _handler) => { },
            post: (_path, _handler) => { },
            put: (_path, _handler) => { },
            delete: (_path, _handler) => { },
            listen: (_port, _callback) => { }
        };
        // Emit app-created event
        const event = {
            type: 'adapter-event',
            timestamp: new Date(),
            source: this.framework,
            data: { type: 'app-created', framework: this.framework }
        };
        this.emit('adapter-event', event);
        return app;
    }
    async applyMiddleware(app) {
        const middlewares = this.getMiddleware();
        // If real Express app, apply middleware properly
        if (this.expressModule && app.use) {
            for (const mw of middlewares) {
                // Create Express-compatible middleware
                const expressMiddleware = (req, res, next) => {
                    // Create context for our middleware
                    const context = {
                        req,
                        res,
                        next: () => {
                            // Call the actual Express next()
                            next();
                        },
                        app
                    };
                    // Run our middleware handler
                    mw.handler(context).then(() => {
                        // Only call next if response wasn't already sent and next wasn't called in handler
                        if (!res.headersSent && !res.finished) {
                            // The handler should have called next() if it wants to continue
                            // We don't call it again here
                        }
                    }).catch((error) => {
                        // Pass errors to Express error handler
                        next(error);
                    });
                };
                // Apply as Express middleware
                app.use(expressMiddleware);
            }
        }
        else {
            // Stub implementation
            for (const mw of middlewares) {
                app.use(async (req, res, next) => {
                    try {
                        await mw.handler({ req, res, next, app });
                        if (next)
                            next();
                    }
                    catch (error) {
                        if (next)
                            next(error);
                    }
                });
            }
        }
    }
    async registerRoute(app, route) {
        const validation = this.validateRoute(route);
        if (!validation.valid) {
            throw new Error(`Invalid route: ${validation.errors?.map(e => e.message).join(', ')}`);
        }
        const method = route.method.toLowerCase();
        const routeHandler = this.expressModule
            ? this.createExpressRouteHandler(route)
            : this.createStubRouteHandler(route);
        if (typeof app[method] === 'function') {
            app[method](route.path, routeHandler);
        }
        this.routes.push(route);
        this.metrics.routesRegistered++;
    }
    createExpressRouteHandler(route) {
        return async (req, res) => {
            try {
                const result = await route.handler(req, res);
                if (!res.headersSent) {
                    res.json(result);
                }
            }
            catch (error) {
                if (!res.headersSent) {
                    res.status(500).json({
                        error: error instanceof Error ? error.message : 'Internal server error'
                    });
                }
            }
        };
    }
    createStubRouteHandler(route) {
        return (_req, _res) => {
            // Stub implementation
            const event = {
                type: 'route-registered',
                timestamp: new Date(),
                source: this.framework,
                data: {
                    method: route.method,
                    path: route.path
                }
            };
            this.emit('route-registered', event);
        };
    }
}
exports.ExpressAdapter = ExpressAdapter;
/**
 * Fastify Framework Adapter
 */
class FastifyAdapter extends FrameworkAdapter {
    constructor() {
        super(...arguments);
        this.framework = 'fastify';
        this.version = '1.0.0';
        this.metadata = {
            name: 'fastify-adapter',
            version: '1.0.0',
            framework: 'fastify',
            description: 'Fastify framework adapter',
            features: ['middleware', 'routing', 'validation', 'serialization'],
            dependencies: ['fastify']
        };
        this.capabilities = {
            middleware: true,
            routing: true,
            validation: true,
            errorHandling: true,
            logging: true,
            metrics: true,
            healthChecks: true
        };
    }
    async createApp() {
        // In a real implementation, this would create a Fastify app
        return {
            addHook: (_hook, _handler) => { },
            route: (_options) => { },
            listen: (_port, _callback) => { },
            register: (_plugin, _options) => { }
        };
    }
    async applyMiddleware(app) {
        const middlewares = this.getMiddleware();
        for (const mw of middlewares) {
            app.register(async (fastify, _options) => {
                fastify.addHook('preHandler', async (request, reply) => {
                    await mw.handler({ request, reply, app });
                });
            });
        }
    }
    async registerRoute(app, route) {
        const validation = this.validateRoute(route);
        if (!validation.valid) {
            const errorMessages = validation.errors?.map(e => e.message) || [];
            throw new Error(`Invalid route: ${errorMessages.join(', ')}`);
        }
        app.route({
            method: route.method,
            url: route.path,
            handler: async (request, reply) => {
                const result = await route.handler(request, reply);
                return result; // Add explicit return
            }
        });
    }
}
exports.FastifyAdapter = FastifyAdapter;
/**
 * Koa Framework Adapter
 */
class KoaAdapter extends FrameworkAdapter {
    constructor() {
        super(...arguments);
        this.framework = 'koa';
        this.version = '1.0.0';
        this.metadata = {
            name: 'koa-adapter',
            version: '1.0.0',
            framework: 'koa',
            description: 'Koa.js framework adapter',
            features: ['middleware', 'routing', 'context', 'async-await'],
            dependencies: ['koa', '@koa/router']
        };
        this.capabilities = {
            middleware: true,
            routing: true,
            validation: true,
            errorHandling: true,
            logging: true,
            metrics: true,
            healthChecks: true
        };
    }
    async createApp() {
        // In a real implementation, this would create a Koa app
        return {
            use: (_middleware) => { },
            listen: (_port, _callback) => { },
            context: {}
        };
    }
    async applyMiddleware(app) {
        const middlewares = this.getMiddleware();
        for (const mw of middlewares) {
            app.use(async (ctx, next) => {
                const context = {
                    req: ctx.request,
                    res: ctx.response,
                    app,
                    next,
                    // Add more context properties as needed
                };
                await mw.handler(context);
                await next();
            });
        }
    }
    async registerRoute(app, route) {
        const validation = this.validateRoute(route);
        if (!validation.valid) {
            const errorMessages = validation.errors?.map(e => e.message) || [];
            throw new Error(`Invalid route: ${errorMessages.join(', ')}`);
        }
        // In a real implementation, this would use @koa/router
        app.use(async (ctx, next) => {
            if (ctx.request.method === route.method && ctx.request.path === route.path) {
                const result = await route.handler(ctx.request, ctx.response);
                ctx.response.body = result;
            }
            else {
                await next();
            }
        });
    }
}
exports.KoaAdapter = KoaAdapter;
/**
 * Adapter Registry for managing multiple framework adapters
 */
class AdapterRegistry extends events_1.EventEmitter {
    constructor() {
        super();
        this.adapters = new Map();
        this.factories = new Map();
        this.isEnabled = true;
        this.registerDefaultFactories();
    }
    static getInstance() {
        if (!AdapterRegistry.instance) {
            AdapterRegistry.instance = new AdapterRegistry();
        }
        return AdapterRegistry.instance;
    }
    registerDefaultFactories() {
        this.factories.set('express', () => new ExpressAdapter({ framework: 'express' }));
        this.factories.set('fastify', () => new FastifyAdapter({ framework: 'fastify' }));
        this.factories.set('koa', () => new KoaAdapter({ framework: 'koa' }));
    }
    // Adapter management
    register(adapter) {
        if (!this.isEnabled) {
            throw new Error('Registry is disabled');
        }
        this.adapters.set(adapter.framework, adapter);
        // Forward adapter events
        adapter.on('adapter-event', (event) => {
            this.emit('adapter-event', event);
        });
        this.emit('adapter-registered', { framework: adapter.framework, adapter });
    }
    unregister(framework) {
        const adapter = this.adapters.get(framework);
        if (adapter) {
            adapter.removeAllListeners();
            this.adapters.delete(framework);
            this.emit('adapter-unregistered', { framework });
            return true;
        }
        return false;
    }
    get(framework) {
        return this.adapters.get(framework);
    }
    create(framework, config) {
        const factory = this.factories.get(framework);
        if (!factory) {
            throw new Error(`No factory registered for framework: ${framework}`);
        }
        const adapter = factory();
        if (config) {
            adapter.updateConfig(config);
        }
        return adapter;
    }
    has(framework) {
        return this.adapters.has(framework);
    }
    supports(framework) {
        return this.factories.has(framework);
    }
    getAll() {
        return Array.from(this.adapters.values());
    }
    getSupportedFrameworks() {
        return Array.from(this.factories.keys());
    }
    getRegisteredFrameworks() {
        return Array.from(this.adapters.keys());
    }
    // Bulk operations
    async healthCheckAll() {
        const results = {};
        for (const [framework, adapter] of this.adapters) {
            try {
                results[framework] = await adapter.healthCheck();
            }
            catch (error) {
                results[framework] = {
                    status: 'unhealthy',
                    timestamp: new Date(),
                    checks: { error: false },
                    framework
                };
            }
        }
        return results;
    }
    // Registry management
    clear() {
        for (const adapter of this.adapters.values()) {
            adapter.removeAllListeners();
        }
        this.adapters.clear();
        this.emit('registry-cleared');
    }
    size() {
        return this.adapters.size;
    }
    // Enable/disable functionality
    enable() {
        this.isEnabled = true;
    }
    disable() {
        this.isEnabled = false;
    }
    isRegistryEnabled() {
        return this.isEnabled;
    }
}
exports.AdapterRegistry = AdapterRegistry;
/**
 * Framework Adapter Manager - Main orchestrator
 */
class AdapterManager extends events_1.EventEmitter {
    constructor(config = { framework: 'express' }) {
        super();
        this.isEnabled = true;
        this.config = config;
        this.registry = AdapterRegistry.getInstance();
        this.setupEventForwarding();
    }
    setupEventForwarding() {
        this.registry.on('adapter-event', (event) => {
            this.emit('event', event);
        });
        this.registry.on('adapter-registered', (data) => {
            this.emit('adapter-registered', data);
        });
        this.registry.on('adapter-unregistered', (data) => {
            this.emit('adapter-unregistered', data);
        });
    }
    // Adapter management
    createAdapter(framework, config) {
        if (!this.isEnabled) {
            throw new Error('Manager is disabled');
        }
        return this.registry.create(framework, config);
    }
    registerAdapter(adapter) {
        if (!this.isEnabled) {
            throw new Error('Manager is disabled');
        }
        this.registry.register(adapter);
    }
    getAdapter(framework) {
        return this.registry.get(framework);
    }
    getSupportedFrameworks() {
        return this.registry.getSupportedFrameworks();
    }
    getRegisteredFrameworks() {
        return this.registry.getRegisteredFrameworks();
    }
    // Health and monitoring
    async performHealthCheck() {
        return this.registry.healthCheckAll();
    }
    async getOverallHealth() {
        const frameworkHealth = await this.performHealthCheck();
        const healthyCount = Object.values(frameworkHealth).filter(h => h.status === 'healthy').length;
        const totalCount = Object.keys(frameworkHealth).length;
        const status = totalCount === 0 ? 'unhealthy' :
            healthyCount === totalCount ? 'healthy' :
                healthyCount > 0 ? 'degraded' : 'unhealthy';
        return {
            status,
            timestamp: new Date(),
            checks: {
                adaptersRegistered: totalCount > 0,
                allAdaptersHealthy: healthyCount === totalCount,
                managerEnabled: this.isEnabled
            },
            metrics: {
                requestsHandled: 0,
                middlewareExecutions: 0,
                routesRegistered: 0,
                errorsEncountered: 0,
                averageResponseTime: 0,
                uptime: 0
            }
        };
    }
    // Configuration
    updateConfig(newConfig) {
        this.config = { ...this.config, ...newConfig };
        this.emit('configuration-updated', { config: this.config });
    }
    getConfig() {
        return { ...this.config };
    }
    // Enable/disable functionality
    enable() {
        this.isEnabled = true;
        this.registry.enable();
    }
    disable() {
        this.isEnabled = false;
        this.registry.disable();
    }
    isManagerEnabled() {
        return this.isEnabled;
    }
    // Cleanup
    async destroy() {
        this.registry.clear();
        this.removeAllListeners();
    }
}
exports.AdapterManager = AdapterManager;
//# sourceMappingURL=adapter-manager.js.map