"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FrameworkAdaptersSDK = void 0;
const events_1 = require("events");
const adapter_manager_1 = require("./adapter-manager");
/**
 * Enhanced Framework Adapters SDK with integrations
 */
class FrameworkAdaptersSDK extends events_1.EventEmitter {
    constructor(options = {}) {
        super();
        this.adapters = new Map();
        this.options = options;
        // Initialize integrations
        this.eventBus = options.eventBus;
        this.logger = options.logger;
        this.cache = options.cache;
        this.config = options.config;
        this.security = options.security;
        this.realtime = options.realtime;
        // Load config if available
        if (this.config?.has('adapters')) {
            const configOptions = this.config.get('adapters');
            this.options = { ...this.options, ...configOptions };
        }
        // Create adapter manager with integrations
        this.manager = new adapter_manager_1.AdapterManager({
            framework: this.options.defaultFramework || 'express',
            enableMetrics: this.options.enableMetrics ?? true,
            enableLogging: this.options.enableLogging ?? true
        });
        this.setupManagerEventForwarding();
    }
    async initialize() {
        // Initialize components
        // await this.manager.initialize(); // Remove this line as initialize() doesn't exist on AdapterManager
        this.log('info', 'SDK Adapters initialized', {
            integrations: {
                eventBus: !!this.eventBus,
                logger: !!this.logger,
                cache: !!this.cache,
                config: !!this.config,
                security: !!this.security,
                realtime: !!this.realtime
            }
        });
        // Emit initialization event
        this.emitEvent('sdk.initialized', {
            timestamp: new Date(),
            source: 'sdk-adapters'
        });
        // Subscribe to external events if event bus is available
        if (this.eventBus) {
            this.subscribeToExternalEvents();
        }
        // Setup WebSocket support if realtime is available
        if (this.realtime && this.options.enableWebSocket) {
            this.setupRealtimeSupport();
        }
    }
    createAdapter(name, config) {
        let adapter;
        // Inject integrations into adapter
        const enhancedConfig = this.enhanceConfigWithIntegrations(config);
        switch (config.framework) {
            case 'express':
                adapter = new adapter_manager_1.ExpressAdapter(enhancedConfig);
                break;
            case 'fastify':
                adapter = new adapter_manager_1.FastifyAdapter(enhancedConfig);
                break;
            case 'koa':
                adapter = new adapter_manager_1.KoaAdapter(enhancedConfig);
                break;
            default:
                throw new Error(`Unsupported framework: ${config.framework}`);
        }
        // Setup adapter event forwarding
        this.setupAdapterEventForwarding(adapter);
        // Store adapter
        this.adapters.set(name, adapter);
        this.log('info', `${config.framework} adapter created`, {
            name,
            framework: config.framework
        });
        // Emit adapter created event
        this.emitEvent('adapter.created', {
            name,
            framework: config.framework,
            timestamp: new Date()
        });
        return adapter;
    }
    createMiddleware(type, options = {}) {
        // Import MiddlewareHelpers from index
        const { MiddlewareHelpers } = require('./index');
        // Enhance middleware with integrations
        const enhancedOptions = {
            ...options,
            cache: this.cache,
            security: this.security,
            logger: this.logger
        };
        // Special handling for rate limit middleware with cache
        if (type === 'rateLimit' && this.cache) {
            return this.createCachedRateLimitMiddleware(enhancedOptions);
        }
        // Special handling for security middleware
        if (type === 'security' && this.security && options.enhanced) {
            return this.createEnhancedSecurityMiddleware(enhancedOptions);
        }
        return MiddlewareHelpers[type]?.(enhancedOptions) || {
            name: type,
            handler: async () => { },
            enabled: false,
            config: options
        };
    }
    createCachedRateLimitMiddleware(options) {
        const cache = this.cache;
        const windowMs = options.windowMs || 15 * 60 * 1000;
        const max = options.max || 100;
        return {
            name: 'rateLimit',
            handler: async (context) => {
                const { req, res, next } = context;
                if (!req || !res) {
                    next?.();
                    return;
                }
                const key = `rateLimit:${req.headers['x-forwarded-for'] || req.socket?.remoteAddress || 'unknown'}`;
                // Get current count from cache
                let record = await cache.get(key);
                const now = Date.now();
                if (!record || record.resetTime < now) {
                    record = { count: 0, resetTime: now + windowMs };
                }
                record.count++;
                // Save to cache
                await cache.set(key, record, Math.ceil(windowMs / 1000));
                // Set headers
                res.setHeader('X-RateLimit-Limit', max.toString());
                res.setHeader('X-RateLimit-Remaining', Math.max(0, max - record.count).toString());
                res.setHeader('X-RateLimit-Reset', new Date(record.resetTime).toISOString());
                if (record.count > max) {
                    res.statusCode = 429;
                    res.setHeader('Retry-After', Math.ceil((record.resetTime - now) / 1000).toString());
                    res.setHeader('Content-Type', 'application/json');
                    const responseData = {
                        error: 'Too Many Requests',
                        message: options.message || 'Rate limit exceeded',
                        retryAfter: Math.ceil((record.resetTime - now) / 1000)
                    };
                    if (typeof res.json === 'function') {
                        res.status(429).json(responseData);
                    }
                    else {
                        res.end(JSON.stringify(responseData));
                    }
                    return;
                }
                next?.();
            },
            order: options.order || 4,
            enabled: options.enabled ?? true,
            config: options
        };
    }
    createEnhancedSecurityMiddleware(options) {
        const security = this.security;
        return {
            name: 'security',
            handler: async (context) => {
                const { req, res, next } = context;
                if (!req) {
                    next?.();
                    return;
                }
                try {
                    // Validate request
                    const isValid = await security.validateRequest(req);
                    if (!isValid) {
                        if (res) {
                            res.statusCode = 403;
                            res.end(JSON.stringify({ error: 'Forbidden' }));
                        }
                        return;
                    }
                    // Sanitize input
                    if (req.body) {
                        req.body = security.sanitizeInput(req.body);
                    }
                    if (req.query) {
                        req.query = security.sanitizeInput(req.query);
                    }
                    next?.();
                }
                catch (error) {
                    this.log('error', 'Security middleware error', error);
                    if (res) {
                        res.statusCode = 500;
                        res.end(JSON.stringify({ error: 'Security check failed' }));
                    }
                }
            },
            order: options.order || 1,
            enabled: options.enabled ?? true,
            config: options
        };
    }
    enhanceConfigWithIntegrations(config) {
        return {
            ...config,
            // Add integration references to config
            ...(this.logger && { logger: this.logger }),
            ...(this.cache && { cache: this.cache }),
            ...(this.security && { security: this.security }),
            ...(this.eventBus && { eventBus: this.eventBus })
        };
    }
    setupAdapterEventForwarding(adapter) {
        // Forward adapter events through event bus
        const eventTypes = [
            'adapter-initialized',
            'route-registered',
            'middleware-added',
            'app-created',
            'error'
        ];
        eventTypes.forEach(eventType => {
            adapter.on(eventType, (data) => {
                this.emitEvent(`adapter.${eventType.replace('-', '.')}`, {
                    ...(typeof data === 'object' && data !== null ? data : { data }),
                    framework: adapter.framework,
                    timestamp: new Date()
                });
            });
        });
    }
    setupManagerEventForwarding() {
        this.manager.on('adapter-registered', (data) => {
            this.emitEvent('manager.adapter.registered', data);
        });
        this.manager.on('error', (error) => {
            this.log('error', 'Adapter manager error', error);
            this.emitEvent('manager.error', { error: error.message });
        });
    }
    subscribeToExternalEvents() {
        if (!this.eventBus)
            return;
        // Listen for gateway requests
        this.eventBus.on('gateway:request.received', (event) => {
            this.handleGatewayRequest(event);
        });
        // Listen for configuration changes
        this.eventBus.on('config:adapters.updated', (event) => {
            this.handleConfigUpdate(event);
        });
        // Listen for auth events
        this.eventBus.on('auth:user.authenticated', (event) => {
            this.log('debug', 'User authenticated', event);
        });
    }
    setupRealtimeSupport() {
        if (!this.realtime)
            return;
        // Create namespace for adapters
        const namespace = this.realtime.createNamespace('/adapters');
        // Setup event forwarding to WebSocket clients
        this.on('adapter.created', (data) => {
            namespace.emit('adapter:created', data);
        });
        this.on('route-registered', (data) => {
            namespace.emit('route:registered', data);
        });
    }
    handleGatewayRequest(event) {
        this.log('debug', 'Gateway request received', event);
        // Route to appropriate adapter based on request
    }
    handleConfigUpdate(event) {
        this.log('debug', 'Configuration updated', event);
        // Update internal configuration
        if (this.config && typeof event === 'object' && event !== null) {
            const { key } = event;
            if (key.startsWith('adapters.')) {
                // Update relevant configuration
            }
        }
    }
    emitEvent(eventType, data) {
        // Emit locally
        this.emit(eventType, data);
        // Emit through event bus if available
        if (this.eventBus) {
            this.eventBus.emit(`adapters:${eventType}`, data);
        }
    }
    log(level, message, meta) {
        if (this.logger) {
            this.logger[level](message, meta);
        }
        else if (this.options.enableLogging) {
            // Fallback to console if no logger
            console[level](`[sdk-adapters] ${message}`, meta);
        }
    }
    getAdapter(name) {
        return this.adapters.get(name);
    }
    getAllAdapters() {
        return new Map(this.adapters);
    }
    getManager() {
        return this.manager;
    }
    async destroy() {
        // Clean up all adapters
        for (const [name, adapter] of this.adapters) {
            adapter.destroy();
            this.adapters.delete(name);
        }
        // Clean up manager
        this.manager.destroy();
        // Emit shutdown event
        this.emitEvent('sdk.destroyed', {
            timestamp: new Date(),
            source: 'sdk-adapters'
        });
        this.log('info', 'SDK Adapters destroyed');
    }
}
exports.FrameworkAdaptersSDK = FrameworkAdaptersSDK;
//# sourceMappingURL=framework-adapters-sdk.js.map