/**
 * Clean Type Definitions for Framework Adapters SDK
 * @iota-big3/sdk-adapters
 */
export interface AdapterConfig {
    framework: FrameworkType;
    version?: string;
    environment?: string;
    port?: number;
    logLevel?: 'debug' | 'info' | 'warn' | 'error';
    enableMetrics?: boolean;
    enableLogging?: boolean;
    options?: Record<string, unknown>;
    customSettings?: Record<string, unknown>;
}
export type FrameworkType = 'express' | 'fastify' | 'koa' | 'hapi' | 'nextjs' | 'react' | 'vue' | 'angular' | 'nestjs' | 'nuxt' | 'svelte';
export interface Middleware {
    name: string;
    handler: MiddlewareHandler;
    order?: number;
    enabled?: boolean;
    config?: Record<string, unknown>;
}
export type MiddlewareHandler = (context: MiddlewareContext) => Promise<any> | any;
export type ExpressMiddleware = (req: GenericRequest, res: GenericResponse, next?: GenericNext) => void | Promise<void>;
export type ExpressRouteHandler = (req: GenericRequest, res: GenericResponse) => void | Promise<void>;
export type FastifyHandler = (request: GenericRequest, reply: GenericResponse) => void | Promise<void>;
export type KoaMiddleware = (ctx: any, next: () => Promise<void>) => void | Promise<void>;
export type GenericHandler = (...args: any[]) => any;
export interface ExpressLikeApp {
    use: (handler: ExpressMiddleware) => void;
    get: (path: string, handler: ExpressRouteHandler) => void;
    post: (path: string, handler: ExpressRouteHandler) => void;
    put: (path: string, handler: ExpressRouteHandler) => void;
    delete: (path: string, handler: ExpressRouteHandler) => void;
    listen: (port: number, callback?: GenericHandler) => void;
}
export interface FastifyLikeApp {
    addHook: (hook: string, handler: FastifyHandler) => void;
    route: (options: any) => void;
    listen: (port: number, callback?: GenericHandler) => void;
    register: (plugin: GenericHandler, options?: any) => void;
}
export interface KoaLikeApp {
    use: (middleware: KoaMiddleware) => void;
    listen: (port: number, callback?: GenericHandler) => void;
    context: Record<string, unknown>;
}
export type FrameworkApp = ExpressLikeApp | FastifyLikeApp | KoaLikeApp;
export interface MiddlewareContext {
    request?: any;
    response?: any;
    req?: any;
    res?: any;
    next?: () => void | Promise<void>;
    app?: any;
    [key: string]: unknown;
}
export interface RouteHandler {
    method: HTTPMethod;
    path: string;
    handler: RouteHandlerFunction;
    middleware?: string[];
    validation?: RouteValidation;
    metadata?: RouteMetadata;
}
export type HTTPMethod = 'GET' | 'POST' | 'PUT' | 'DELETE' | 'PATCH' | 'HEAD' | 'OPTIONS';
export interface FastifyRouteOptions {
    method: HTTPMethod | HTTPMethod[];
    url: string;
    handler: FastifyHandler;
    schema?: Record<string, unknown>;
    preHandler?: FastifyHandler | FastifyHandler[];
    config?: Record<string, unknown>;
}
export interface PluginOptions {
    prefix?: string;
    logLevel?: string;
    [key: string]: unknown;
}
export type HandlerResponse = Record<string, unknown> | string | number | boolean | null | undefined;
export type RouteHandlerFunction = (request: GenericRequest, response: GenericResponse) => Promise<HandlerResponse> | HandlerResponse;
export interface RouteValidation {
    params?: ValidationSchema;
    query?: ValidationSchema;
    body?: ValidationSchema;
    headers?: ValidationSchema;
}
export interface ValidationSchema {
    [key: string]: {
        type: 'string' | 'number' | 'boolean' | 'object' | 'array';
        required?: boolean;
        pattern?: string;
        min?: number;
        max?: number;
    };
}
export interface RouteMetadata {
    description?: string;
    tags?: string[];
    deprecated?: boolean;
    version?: string;
}
export interface AdapterMetadata {
    name: string;
    version: string;
    framework: FrameworkType;
    description: string;
    features: string[];
    dependencies?: string[];
}
export interface AdapterCapabilities {
    middleware: boolean;
    routing: boolean;
    validation: boolean;
    errorHandling: boolean;
    logging: boolean;
    metrics: boolean;
    healthChecks: boolean;
}
export interface FrameworkConfig {
    port?: number;
    host?: string;
    cors?: CORSConfig;
    security?: SecurityConfig;
    logging?: LoggingConfig;
    middleware?: MiddlewareConfig;
    routing?: RoutingConfig;
}
export interface CORSConfig {
    enabled: boolean;
    origin?: string | string[] | boolean;
    methods?: string[];
    allowedHeaders?: string[];
    credentials?: boolean;
}
export interface SecurityConfig {
    helmet?: boolean;
    rateLimit?: RateLimitConfig;
    authentication?: AuthConfig;
    authorization?: AuthzConfig;
}
export interface RateLimitConfig {
    enabled: boolean;
    windowMs?: number;
    max?: number;
    message?: string;
}
export interface AuthConfig {
    enabled: boolean;
    strategy?: 'jwt' | 'session' | 'oauth' | 'basic';
    secret?: string;
    options?: Record<string, unknown>;
}
export interface AuthzConfig {
    enabled: boolean;
    roles?: string[];
    permissions?: string[];
    rules?: AuthzRule[];
}
export interface AuthzRule {
    resource: string;
    action: string;
    roles: string[];
}
export interface LoggingConfig {
    enabled: boolean;
    level?: string;
    format?: 'json' | 'text';
    destination?: 'console' | 'file' | 'remote';
    options?: Record<string, unknown>;
}
export interface MiddlewareConfig {
    enabled: boolean;
    order?: string[];
    custom?: Record<string, unknown>;
}
export interface RoutingConfig {
    prefix?: string;
    versioning?: boolean;
    caseSensitive?: boolean;
    strictRouting?: boolean;
}
export interface HealthCheckResult {
    status: 'healthy' | 'degraded' | 'unhealthy';
    timestamp?: Date;
    framework?: FrameworkType;
    version?: string;
    uptime?: number;
    metrics?: AdapterMetrics;
    checks?: Record<string, boolean>;
    details?: {
        middlewareCount?: number;
        routeCount?: number;
        enabled?: boolean;
        initialized?: boolean;
    };
}
export interface AdapterMetrics {
    requestsHandled: number;
    middlewareExecutions: number;
    routesRegistered: number;
    errorsEncountered: number;
    averageResponseTime: number;
    uptime: number;
}
export interface AdapterEvent {
    type: AdapterEventType;
    timestamp: Date;
    source: string;
    data: unknown;
    metadata?: Record<string, unknown>;
}
export type AdapterEventType = 'adapter-initialized' | 'adapter-destroyed' | 'middleware-added' | 'middleware-removed' | 'route-registered' | 'route-removed' | 'request-handled' | 'error-occurred' | 'configuration-updated';
export interface AdapterError {
    code: string;
    message: string;
    details?: unknown;
    stack?: string;
    timestamp: Date;
}
export type ErrorSeverity = 'low' | 'medium' | 'high' | 'critical';
export interface AdapterPlugin {
    name: string;
    version: string;
    install: (adapter: any) => Promise<void> | void;
    uninstall?: (adapter: any) => Promise<void> | void;
    config?: Record<string, unknown>;
}
export interface AdapterFactory {
    create: (config: AdapterConfig) => Promise<any>;
    supports: (framework: FrameworkType) => boolean;
    getCapabilities: (framework: FrameworkType) => AdapterCapabilities;
}
export interface ExpressConfig extends AdapterConfig {
    framework: 'express';
    port?: number;
    host?: string;
    cors?: CORSConfig;
    security?: SecurityConfig;
    logging?: LoggingConfig;
    middleware?: MiddlewareConfig;
    routing?: RoutingConfig;
    trustProxy?: boolean;
    jsonLimit?: string;
    urlEncodedLimit?: string;
    staticFiles?: StaticFilesConfig;
}
export interface FastifyConfig extends AdapterConfig {
    framework: 'fastify';
    port?: number;
    host?: string;
    cors?: CORSConfig;
    security?: SecurityConfig;
    logging?: LoggingConfig;
    middleware?: MiddlewareConfig;
    routing?: RoutingConfig;
    logger?: boolean | object;
    disableRequestLogging?: boolean;
    maxParamLength?: number;
    bodyLimit?: number;
}
export interface KoaConfig extends AdapterConfig {
    framework: 'koa';
    port?: number;
    host?: string;
    cors?: CORSConfig;
    security?: SecurityConfig;
    logging?: LoggingConfig;
    middleware?: MiddlewareConfig;
    routing?: RoutingConfig;
    keys?: string[];
    proxy?: boolean;
    subdomainOffset?: number;
    proxyIpHeader?: string;
    maxIpsCount?: number;
}
export interface StaticFilesConfig {
    enabled: boolean;
    path: string;
    options?: Record<string, unknown>;
}
export type ConfigurationType<T extends FrameworkType> = T extends 'express' ? ExpressConfig : T extends 'fastify' ? FastifyConfig : T extends 'koa' ? KoaConfig : FrameworkConfig;
export interface AdapterRegistry {
    register: (framework: FrameworkType, factory: AdapterFactory) => void;
    unregister: (framework: FrameworkType) => boolean;
    get: (framework: FrameworkType) => AdapterFactory | undefined;
    list: () => FrameworkType[];
    supports: (framework: FrameworkType) => boolean;
}
export interface RequestContext {
    method: HTTPMethod;
    path: string;
    query: Record<string, any>;
    params: Record<string, any>;
    body: any;
    headers: Record<string, string>;
    user?: any;
    session?: any;
    [key: string]: any;
}
export interface ResponseContext {
    status: number;
    headers: Record<string, string>;
    body: any;
    sent: boolean;
}
export interface ValidationResult {
    valid: boolean;
    errors?: ValidationError[];
    warnings?: ValidationError[];
}
export interface ValidationError {
    field: string;
    message: string;
    code?: string;
    path?: string;
    value?: unknown;
}
export interface ValidationWarning {
    field: string;
    message: string;
    suggestion?: string;
}
export interface GenericRequest {
    method?: string;
    path?: string;
    url?: string;
    headers?: Record<string, string | string[]>;
    body?: unknown;
    query?: Record<string, unknown>;
    params?: Record<string, string>;
    [key: string]: unknown;
}
export interface GenericResponse {
    status: (code: number) => GenericResponse;
    json: (data: unknown) => void;
    send: (data: unknown) => void;
    header: (name: string, value: string) => GenericResponse;
    [key: string]: unknown;
}
export interface GenericNext {
    (error?: Error | unknown): void;
}
export interface KoaContext {
    request: GenericRequest;
    response: GenericResponse;
    state: Record<string, unknown>;
    cookies: {
        get: (name: string) => string | undefined;
        set: (name: string, value: string, options?: Record<string, unknown>) => void;
    };
    throw: (status: number, message?: string) => never;
    assert: (value: unknown, status: number, message?: string) => void;
    [key: string]: unknown;
}
//# sourceMappingURL=types.d.ts.map